#include "stdafx.h"
#include "resource.h"
#include "MenuSpawn.h"
//#include "comctl32.h"

#define NORMALCOLOR_SELECT

#define RGB_BUTTON_BLACK    (GetSysColor(COLOR_WINDOWFRAME))
#define RGB_BUTTON_WHITE    (GetSysColor(COLOR_BTNHIGHLIGHT))
#define RGB_BUTTON_LIGHT    (GetSysColor(COLOR_BTNFACE))
#define RGB_BUTTON_DARK     (GetSysColor(COLOR_BTNSHADOW))
#define RGB_MENU_FACE       (GetSysColor(COLOR_MENU))
#define RGB_MENUTEXT_BACK   (GetSysColor(COLOR_HIGHLIGHT))
#define RGB_MENU_TEXT       (GetSysColor(COLOR_MENUTEXT))


void DrawGradient(HDC hDC,const RECT &rectClient,COLORREF m_clrStart,COLORREF m_clrEnd)
{
	HBRUSH brush;	// Brush to fill in the bar	
	if(m_clrEnd == m_clrStart)
	{
		if((brush = CreateSolidBrush(m_clrEnd)) == NULL)
			return;
		FillRect(hDC,&rectClient,brush);
		DeleteObject(brush);
		return;
	}
	int nMaxWidth = rectClient.right-rectClient.left;

	RECT rectFill;			   // Rectangle for filling band
	int r, g, b;
	float rStep, gStep, bStep,fStep;

	r = (GetRValue(m_clrEnd) - GetRValue(m_clrStart));
	g = (GetGValue(m_clrEnd) - GetGValue(m_clrStart));
	b =  (GetBValue(m_clrEnd) - GetBValue(m_clrStart));

	// Make the number of steps equal to the greatest distance
	int nSteps = max(abs(r), max(abs(g), abs(b)));

	fStep = (float)rectClient.right / (float)nSteps;

	// Calculate the step size for each color
	rStep = r/(float)nSteps;
	gStep = g/(float)nSteps;
	bStep = b/(float)nSteps;

	// Reset the colors to the starting position
	r = GetRValue(m_clrStart);
	g = GetGValue(m_clrStart);
	b = GetBValue(m_clrStart);

	// Start filling bands
	for (int iOnBand = 0; iOnBand < nSteps; iOnBand++) 
	{
		INT nLeft  = rectClient.left+(INT)(iOnBand*fStep);
		INT nRight = nLeft+(INT)((iOnBand+1)*fStep);
		::SetRect(&rectFill,
			nLeft,
			rectClient.top,
			nRight,
			rectClient.bottom
			);
	
		brush = CreateSolidBrush(RGB(r+rStep*iOnBand,g + gStep*iOnBand, b + bStep *iOnBand));
		FillRect(hDC,&rectFill,brush);
		DeleteObject(brush);

		//if (nRight >= rectFill.right)
		//	return;
	}
}
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CMenuSpawn::CMenuSpawn()
{
	ilList			= NULL;
	ilOther			= NULL;
	bmpBack			= NULL;
	bIsPopup		= FALSE;
	bBackBitmap		= FALSE;
	m_BackImageList		= NULL;
	m_iBackImageWidth	= 0;
	hMenuFont		= NULL;
	hGuiFont		= NULL;
	iSpawnItem		= 0;
	iImageItem		= 0;
}

CMenuSpawn::CMenuSpawn(BOOL _IsPopup)
{
	ilList			= NULL;
	ilOther			= NULL;
	bmpBack			= NULL;
	bIsPopup		= FALSE;
	bBackBitmap		= FALSE;
	m_BackImageList		= NULL;
	m_iBackImageWidth	= 0;
	hMenuFont		= NULL;
	hGuiFont		= NULL;
	iSpawnItem		= 0;
	iImageItem		= 0;

	bIsPopup		= _IsPopup;
}

CMenuSpawn::~CMenuSpawn()
{
	if (iSpawnItem > 0)
	{
		for (int t = 0; t < iSpawnItem; t++)
			if (pSpawnItem[t]) delete pSpawnItem[t];

		GlobalFree((HGLOBAL) pSpawnItem);
	}
	if (iImageItem > 0)
		GlobalFree((HGLOBAL) pImageItem);

	if (hMenuFont) ::DeleteObject((HGDIOBJ)hMenuFont);
	if (hGuiFont) ::DeleteObject((HGDIOBJ)hGuiFont);
	if(bmpBack)
		DeleteObject(bmpBack);
	if(ilList)
		DeleteObject(ilList);
	if(ilOther)
		DeleteObject(ilOther);
	if(m_BackImageList)
		DeleteObject(m_BackImageList);
}

VOID CMenuSpawn::TransparentBlt(HDC hDestDc,int x, int y, int w, int h, HBITMAP hBmp, int sx, int sy, COLORREF crTransparent)
{
	HDC memDC, maskDC, tempDC;
	maskDC = CreateCompatibleDC(hDestDc);
	HBITMAP maskBitmap;
	
	//add these to store return of SelectObject() calls
	HBITMAP pOldMemBmp = NULL;
	HBITMAP pOldMaskBmp = NULL;
	
	memDC = CreateCompatibleDC(hDestDc);
	tempDC = CreateCompatibleDC(hDestDc);
	HBITMAP bmpImage;
	bmpImage = CreateCompatibleBitmap(hDestDc, w, h);
	pOldMemBmp = (HBITMAP)SelectObject(memDC,bmpImage );

	HBITMAP oldBmp = (HBITMAP)SelectObject(tempDC,hBmp);
	
	BitBlt(memDC,0,0,w, h, tempDC, sx, sy, SRCCOPY );
	
	// Create monochrome bitmap for the mask
	maskBitmap = CreateBitmap(w, h, 1, 1, NULL);
	pOldMaskBmp = (HBITMAP)SelectObject(maskDC,maskBitmap );
	SetBkColor(memDC,crTransparent);
	
	// Create the mask from the memory DC
	BitBlt(maskDC,0, 0, w, h, memDC, 0, 0, SRCCOPY);
	
	SetBkColor(memDC,RGB(0,0,0));
	::SetTextColor(memDC,RGB(255,255,255));
	BitBlt(memDC,0, 0, w, h,maskDC, 0, 0, SRCAND);
	
	// Set the foreground to black. See comment above.
	SetBkColor(hDestDc,RGB(255,255,255));
	::SetTextColor(hDestDc,RGB(0,0,0));
	BitBlt(hDestDc,x, y, w, h, maskDC, 0, 0, SRCAND);
	
	// Combine the foreground with the background
	BitBlt(hDestDc,x, y, w, h, memDC, 0, 0, SRCPAINT);
	
	SelectObject(tempDC,oldBmp);
	if (pOldMaskBmp) 
		SelectObject(maskDC,pOldMaskBmp );
	if (pOldMemBmp)  
		SelectObject(memDC,pOldMemBmp );
	DeleteObject(maskBitmap);
	DeleteObject(pOldMaskBmp);
	DeleteObject(pOldMemBmp);
	DeleteObject(bmpImage);
	DeleteDC(memDC);
	DeleteDC(maskDC);
	DeleteDC(tempDC);
}



void CMenuSpawn::Init(HINSTANCE hInstance)
{
	hInst = hInstance;
	crMenuText = GetSysColor(COLOR_MENUTEXT);
	crMenuTextSel = GetSysColor(COLOR_HIGHLIGHTTEXT);

	cr3dFace = GetSysColor(COLOR_3DFACE);
	crMenu = GetSysColor(COLOR_MENU);

	crHighlight = GetSysColor(COLOR_HIGHLIGHT);
	cr3dHilight = GetSysColor(COLOR_3DHILIGHT);
	cr3dShadow = GetSysColor(COLOR_3DSHADOW);
	crGrayText = GetSysColor(COLOR_GRAYTEXT);

	m_clrBtnFace = GetSysColor(COLOR_BTNFACE);
	m_clrBtnHilight = GetSysColor(COLOR_BTNHILIGHT);
	m_clrBtnShadow = GetSysColor(COLOR_BTNSHADOW);

	iSpawnItem = 0;
	pSpawnItem = NULL;

	iImageItem = 0;
	pImageItem = NULL;

	szImage.cx = 20;
	szImage.cy = 20;

	hMenuFont = NULL;
	COLORMAP cMap[3] = { 
		{ RGB(128,128,128), cr3dShadow }, 
		{ RGB(192,192,192), cr3dFace }, 
		{ RGB(255,255,255), cr3dHilight }
	};
	HBITMAP bmp;
	bmp = CreateMappedBitmap(hInst,IDB_MENUCHK, 0, cMap, 3);
	ilOther = ImageList_Create(19,19,ILC_COLOR4|ILC_MASK,1,0);
	ImageList_AddMasked(ilOther,bmp,cr3dFace);
	DeleteObject(bmp);

	NONCLIENTMETRICS ncm;
	memset(&ncm, 0, sizeof(ncm));
	ncm.cbSize = sizeof(ncm);

	::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, (PVOID) &ncm, 0);

	hGuiFont = ::CreateFontIndirect(&ncm.lfMenuFont);
}

VOID CMenuSpawn::SetBarImage(UINT nID,INT cx)
{
	COLORREF  cl = GetSysColor(COLOR_MENU);
	m_BackImageList	= ImageList_LoadBitmap(hInst,MAKEINTRESOURCE(nID),cx,1,cl);
	m_iBackImageWidth = cx;
}
BOOL CMenuSpawn::AddToolBarResource(unsigned int resId)
{
	HRSRC hRsrc = ::FindResource(hInst, MAKEINTRESOURCE(resId), MAKEINTRESOURCE(0xF1));
	if (hRsrc == NULL) return FALSE;

	HGLOBAL hGlb = ::LoadResource(hInst, hRsrc);
	if (hGlb == NULL) return FALSE;

	ToolBarData* pTBData = (ToolBarData*) ::LockResource(hGlb);
	if (pTBData == NULL) return FALSE;
	
	if(pTBData->wVersion != 1) return FALSE;

	HBITMAP bmp;
	bmp = LoadBitmap(hInst,MAKEINTRESOURCE(resId));
	int nBmpItems = ImageList_AddMasked(ilList,bmp,RGB(192,192,192));
	DeleteObject(bmp);

	WORD* pItem = (WORD*)(pTBData+1);
	
	for(int i=0; i<pTBData->wItemCount; i++, pItem++)
	{
		if(*pItem != 0)//ID_SEPARATOR)
			AddImageItem(nBmpItems++, (WORD) *pItem);
	}
// ** it seem that Windows doesn't free these resource (from Heitor Tome)
	UnlockResource(hGlb);
	FreeResource(hGlb);
// **
	return TRUE;
}

BOOL CMenuSpawn::LoadToolBarResource(unsigned int resId)
{
	HRSRC hRsrc = ::FindResource(hInst, MAKEINTRESOURCE(resId),MAKEINTRESOURCE(0xF1));
	if (hRsrc == NULL) return FALSE;

	HGLOBAL hGlb = ::LoadResource(hInst, hRsrc);
	if (hGlb == NULL) return FALSE;

	ToolBarData* pTBData = (ToolBarData*) ::LockResource(hGlb);
	if (pTBData == NULL) return FALSE;
	
	if(pTBData->wVersion != 1) return FALSE;

	szImage.cx = (int) pTBData->wWidth;
	szImage.cy = (int) pTBData->wHeight;

	if ((ilList = ImageList_Create(szImage.cx, szImage.cy, ILC_COLOR4|ILC_MASK, pTBData->wItemCount, 0)) == NULL)
		return FALSE;

	ImageList_SetBkColor(ilList,cr3dFace);

	HBITMAP bmp;
	bmp = LoadBitmap(hInst,MAKEINTRESOURCE(resId));
	ImageList_AddMasked(ilList,bmp,RGB(192,192,192));
	DeleteObject(bmp);

	WORD* pItem = (WORD*)(pTBData+1);
	int nBmpItems = 0;
	for(int i=0; i<pTBData->wItemCount; i++, pItem++)
	{
		if(*pItem != 0)
			AddImageItem(nBmpItems++, (WORD) *pItem);
	}
// ** it seem that Windows doesn't free these resource (from Heitor Tome)
	UnlockResource(hGlb);
	FreeResource(hGlb);
// **
	return TRUE;
}

void CMenuSpawn::AddImageItem(const int idx, WORD cmd)
{
	if (iImageItem == 0)
		pImageItem = (ImageItem *) GlobalAlloc(GPTR, sizeof(ImageItem));
	else
		pImageItem = (ImageItem *) GlobalReAlloc((HGLOBAL) pImageItem, sizeof(ImageItem) * (iImageItem + 1), GMEM_MOVEABLE|GMEM_ZEROINIT);
	
	if(pImageItem == NULL)
		return;
	pImageItem[iImageItem].iCmd = (int) cmd;
	pImageItem[iImageItem].iImageIdx = idx;
	iImageItem ++;
}

VOID CMenuSpawn::RemapMenu(HMENU hMenu)
{
	static int iRecurse = 0;
	iRecurse ++;

	if(!hMenu) return;
	int nItem = GetMenuItemCount(hMenu);
	while ((--nItem)>=0)
	{
		UINT itemId = GetMenuItemID(hMenu,nItem);
		if (itemId == (UINT) -1)
		{
			HMENU pops = GetSubMenu(hMenu,nItem);
			if (pops) 
				RemapMenu(pops);
			if (bIsPopup || iRecurse > 0)
			{
				char cs[100];
				INT count = GetMenuString(hMenu,nItem, cs,100, MF_BYPOSITION);
				if (count > 0)
				{
					SpawnItem * sp = AddSpawnItem(cs, (!bIsPopup && iRecurse == 1) ? -4 : -2);
					ModifyMenu(hMenu,nItem,MF_BYPOSITION|MF_OWNERDRAW, (UINT) -1, (LPCTSTR)sp);
				}
			}
		}
		else
		{
			if (itemId != 0)
			{
				UINT oldState = GetMenuState(hMenu,nItem,MF_BYPOSITION);
				if (!(oldState&MF_OWNERDRAW) && !(oldState&MF_BITMAP))
				{
					if(oldState == (UINT)-1)
						return;
					char cs[100];
					if(GetMenuString(hMenu,nItem, cs, 100,MF_BYPOSITION) > 0)
					{
						SpawnItem * sp = AddSpawnItem(cs, itemId);
						ModifyMenu(hMenu,nItem,MF_BYPOSITION|MF_OWNERDRAW|oldState, (LPARAM)itemId, (LPCTSTR)sp);
					}
				}
			}
			else
			{
				UINT oldState = GetMenuState(hMenu,nItem,MF_BYPOSITION);
				if (!(oldState&MF_OWNERDRAW) && !(oldState&MF_BITMAP))
				{
					if(oldState == (UINT)-1)
						return;
					SpawnItem * sp = AddSpawnItem("--", -3);
					ModifyMenu(hMenu,nItem,MF_BYPOSITION|MF_OWNERDRAW|oldState, (LPARAM)itemId, (LPCTSTR)sp);
				}
			}
		}
	}
	iRecurse --;
}

CMenuSpawn::SpawnItem * CMenuSpawn::AddSpawnItem(const char * txt, const int cmd)
{
	if (iSpawnItem == 0)
		pSpawnItem = (SpawnItem **) GlobalAlloc(GPTR, sizeof(SpawnItem));
	else
		pSpawnItem = (SpawnItem **) GlobalReAlloc((HGLOBAL) pSpawnItem, sizeof(SpawnItem) * (iSpawnItem + 1), GMEM_MOVEABLE|GMEM_ZEROINIT);

	if(!pSpawnItem)
		return NULL;

	SpawnItem * p = new SpawnItem;
	if(p == NULL)
		return NULL;
	pSpawnItem[iSpawnItem] = p;
	lstrcpy(p->cText, txt);
	p->iCmd = cmd;

	if (cmd >= 0) p->iImageIdx = FindImageItem(cmd);
	else p->iImageIdx = cmd;

	iSpawnItem ++;
	return p;
}

int CMenuSpawn::FindImageItem(const int cmd)
{
	for (int t = 0; t < iImageItem; t++)
		if (pImageItem[t].iCmd == cmd) return pImageItem[t].iImageIdx;

	return -1;
}
VOID MyDraw3dRect(HDC hDC,LPCRECT lpRect, COLORREF clrTopLeft, COLORREF clrBottomRight )
{
	RECT rc;
	rc = *lpRect;
	InflateRect(&rc,-1,-1);
	HPEN pnDk,pnLt;
	pnDk = CreatePen(PS_SOLID,1,clrTopLeft);
	pnLt = CreatePen(PS_SOLID,1,clrBottomRight);

	//left
	HPEN opn = (HPEN)SelectObject(hDC,pnDk);
	MoveToEx(hDC,rc.left, rc.bottom,NULL);
	LineTo(hDC,rc.left, rc.top);
	//top
	MoveToEx(hDC,rc.left, rc.top,NULL);
	LineTo(hDC,rc.right, rc.top);

	SelectObject(hDC,pnLt);
	//right
	MoveToEx(hDC,rc.right, rc.top,NULL);
	LineTo(hDC,rc.right, rc.bottom);

	MoveToEx(hDC,rc.right, rc.bottom,NULL);
	LineTo(hDC,rc.left, rc.bottom);

	SelectObject(hDC,opn);
	DeleteObject(pnDk);
	DeleteObject(pnLt);
}
BOOL CMenuSpawn::DrawItem(LPDRAWITEMSTRUCT lp)
{
	BOOL res = FALSE;
	if (lp->CtlType == ODT_MENU)
	{
		UINT id = lp->itemID;
		UINT state = lp->itemState;
		BOOL bEnab = !(state & ODS_DISABLED);
		BOOL bSelect = (state & ODS_SELECTED) ? TRUE : FALSE;
		BOOL bChecked = (state & ODS_CHECKED) ? TRUE : FALSE;

		SpawnItem * pItem = (SpawnItem *) lp->itemData;
		if (pItem)
		{
			HDC hDC = lp->hDC;
			HFONT pft = (HFONT)hMenuFont?hMenuFont:hGuiFont;
			HFONT of = (HFONT)SelectObject(hDC,pft);
			RECT rc;
			rc = lp->rcItem;
			//ͼbarĿ
			if (pItem->iCmd != -4)
				rc.left += m_iBackImageWidth;
			//Ʊͼbar
			if (	lp->itemID != 0 
				&& lp->rcItem.top <= 10 
				&& (lp->itemAction & ODA_DRAWENTIRE)
				&& pItem->iCmd != -4 
				&& pItem->iCmd != -3
				)
				ImageList_Draw(m_BackImageList,0,hDC,lp->rcItem.left,lp->rcItem.top,ILD_NORMAL);

			RECT rcImage,rcText;
			rcImage = rc;
			rcText  = rc;
			rcImage.right = rcImage.left + rc.bottom-rc.top;
			rcImage.bottom = rc.bottom;

			if (bBackBitmap)
			{
				HDC tempDC;
				tempDC = CreateCompatibleDC(hDC);
				HBRUSH myBrush;
				myBrush = CreateSolidBrush(crMenu);
				FillRect(tempDC,&rc,myBrush);
				HBITMAP ob = (HBITMAP)SelectObject(tempDC,bmpBack);
				FillRect(hDC,&rc,myBrush);
				BitBlt(hDC,rc.left, rc.top,rc.right-rc.left,rc.bottom-rc.top,tempDC, rc.left, rc.top, SRCCOPY );
				SelectObject(tempDC,ob);
				DeleteObject(myBrush);
				DeleteDC(tempDC);
			}
			if (pItem->iCmd == -3) // is a separator
			{
				HPEN pnDk;
				pnDk = CreatePen(PS_SOLID,1,cr3dShadow);
				HPEN pnLt;
				pnLt = CreatePen(PS_SOLID,1,cr3dHilight);
				HPEN opn = (HPEN)SelectObject(hDC,pnDk);
				MoveToEx(hDC,rc.left + 2, rc.top + 2,NULL);
				LineTo(hDC,rc.right - 2, rc.top + 2);
				SelectObject(hDC,pnLt);
				MoveToEx(hDC,rc.left + 2, rc.top + 3,NULL);
				LineTo(hDC,rc.right - 2, rc.top + 3);
				SelectObject(hDC,opn);
				DeleteObject(pnDk);
				DeleteObject(pnLt);
			}
			else if (pItem->iCmd == -4) // is a title item
			{
				char cs[200];
				strcpy(cs,pItem->cText);
				RECT rcBdr;
				rcBdr = rcText;
				if (bSelect && bEnab)
				{
					rcText.top ++;
					rcText.left += 2;
				}
//modify for menu color incorrect on Winxp
				//HBRUSH myBrush = CreateSolidBrush(crMenu);
				HBRUSH myBrush = CreateSolidBrush(cr3dFace);
				FillRect(hDC,&rcText,myBrush);
				DeleteObject(myBrush);
				int obk = SetBkMode(hDC,TRANSPARENT);
				DrawText(hDC,cs, strlen(cs),&rcText, DT_VCENTER|DT_CENTER|DT_SINGLELINE);
				if (bSelect && bEnab)
					MyDraw3dRect(hDC,&rcBdr,cr3dShadow,cr3dHilight);
				SetBkMode(hDC,obk);
			}
			else//˵
			{
				rcText.left += (rcImage.right-rcImage.left) + 1;
				int obk = SetBkMode(hDC,TRANSPARENT);
				
				COLORREF ocr;
				if (bSelect)
				{
					HBRUSH myBrush = CreateSolidBrush(crHighlight);
					if (pItem->iImageIdx >= 0 || (state & ODS_CHECKED))
#ifdef	NORMALCOLOR_SELECT
						FillRect(hDC,&rcText, myBrush);
#else
						DrawGradient(hDC,rcText,RGB(255,0,0),RGB(0,255,0));
#endif
					else
#ifdef	NORMALCOLOR_SELECT
						FillRect(hDC,&rc, myBrush);
#else
						DrawGradient(hDC,rc,RGB(255,0,0),RGB(0,255,0));
#endif
					ocr = ::SetTextColor(hDC,crMenuTextSel);
					DeleteObject(myBrush);
				}
				else
				{
					if (!bBackBitmap) 
					{
						HBRUSH myBrush = CreateSolidBrush(crMenu);
						if (pItem->iImageIdx >= 0 || (state & ODS_CHECKED))
							FillRect(hDC,&rcText,myBrush);
						else
							FillRect(hDC,&rc/*rcText*/,myBrush);
						DeleteObject(myBrush);
					}
					ocr = ::SetTextColor(hDC,crMenuText);
				}
				if (pItem->iImageIdx >= 0)
				{
					int ay = ((rcImage.bottom-rcImage.top) - szImage.cy) / 2;
					int ax = ((rcImage.right-rcImage.left) - szImage.cx) / 2;
					if (bSelect && bEnab)
						MyDraw3dRect(hDC,&rcImage,cr3dHilight,cr3dShadow);
					else
					{
						if (!bBackBitmap) 
							MyDraw3dRect(hDC,&rcImage,crMenu,crMenu);
					}
					if (bEnab)
					{
						ImageList_Draw(ilList,pItem->iImageIdx,hDC,rcImage.left+ax,rcImage.top +ay, ILD_NORMAL);
					}
					else
					{
						HICON hIcon = ImageList_ExtractIcon(hInst,ilList,pItem->iImageIdx);
						LPARAM lData = MAKELPARAM((HICON)hIcon,0);
						DrawState(hDC,(HBRUSH)NULL,NULL,lData,0,rcImage.left+ax,rcImage.top+ay, 
							szImage.cx,szImage.cy, DST_ICON | DSS_DISABLED);
						DestroyIcon(hIcon);
					}
				}
				else
				{
					if (bChecked)
					{
						int ay = (rcImage.bottom-rcImage.top - szImage.cy) / 2;
						int ax = (rcImage.right-rcImage.left - szImage.cx) / 2;
						ImageList_Draw(ilOther,0,hDC,rcImage.left+ax, rcImage.top + ay - 2, ILD_NORMAL);
					}
				}
				char cs[200],cs1[200];
				strcpy(cs,pItem->cText);
				SIZE sz;
				GetTextExtentPoint32(hDC,cs,strlen(cs),&sz);
				int ay1 = ((rcText.bottom-rcText.top) - sz.cy) / 2;
				rcText.top += ay1;
				rcText.left += 2;
				rcText.right -= 15;

				char *tf = strstr(cs,"\t");
				//int tf = cs.Find('\t');
				if (tf )
				{
					//cs1 = cs.Right(cs.GetLength() - tf - 1);
					strcpy(cs1,tf+1);
					//cs = cs.Left(tf);
					*tf = 0;
					if (!bEnab)
					{
						if (!bSelect)
						{
							RECT rcText1;
							rcText1 = rcText;
							InflateRect(&rcText1,-1,-1);
							::SetTextColor(hDC,cr3dHilight);
							DrawText(hDC,cs,strlen(cs),&rcText1, DT_VCENTER|DT_LEFT);
							DrawText(hDC,cs1,strlen(cs1),&rcText1, DT_VCENTER|DT_RIGHT);
							::SetTextColor(hDC,crGrayText);
							DrawText(hDC,cs,strlen(cs),&rcText, DT_VCENTER|DT_LEFT);
							DrawText(hDC,cs1,strlen(cs1),&rcText, DT_VCENTER|DT_RIGHT);
						}
						else
						{
							::SetTextColor(hDC,crMenu);
							DrawText(hDC,cs,strlen(cs),&rcText, DT_VCENTER|DT_LEFT);
							DrawText(hDC,cs1, strlen(cs1),&rcText, DT_VCENTER|DT_RIGHT);
						}
					}
					else
					{
						DrawText(hDC,cs,strlen(cs),&rcText, DT_VCENTER|DT_LEFT);
						DrawText(hDC,cs1,strlen(cs1),&rcText, DT_VCENTER|DT_RIGHT);
					}
				}
				else 
				{
					if (!bEnab)
					{
						if (!bSelect)
						{
							RECT rcText1;
							rcText1 = rcText;
							InflateRect(&rcText1,-1,-1);
							::SetTextColor(hDC,cr3dHilight);
							DrawText(hDC,cs,strlen(cs),&rcText1, DT_VCENTER|DT_LEFT|DT_EXPANDTABS);
							::SetTextColor(hDC,crGrayText);
							DrawText(hDC,cs,strlen(cs),&rcText, DT_VCENTER|DT_LEFT|DT_EXPANDTABS);
						}
						else
						{
							::SetTextColor(hDC,crMenu);
							DrawText(hDC,cs, strlen(cs),&rcText, DT_VCENTER|DT_LEFT|DT_EXPANDTABS);
						}
					}
					else
						DrawText(hDC,cs,strlen(cs),&rcText, DT_VCENTER|DT_LEFT|DT_EXPANDTABS);
				}
				::SetTextColor(hDC,ocr);
				SetBkMode(hDC,obk);
			}
			SelectObject(hDC,of);
		}
		res = TRUE;
	}
	return res;
}

BOOL CMenuSpawn::MeasureItem(HWND hMainWnd,LPMEASUREITEMSTRUCT lpm)
{
	BOOL res = FALSE;
	if (lpm->CtlType == ODT_MENU)
	{
		UINT id = lpm->itemID;

		SpawnItem * pItem = (SpawnItem *) lpm->itemData;
		if (pItem)
		{
			if (pItem->iCmd == -3) // is a separator
			{
				lpm->itemWidth  = 10;
				lpm->itemHeight = 6;
			}
			else
			{
				char cs[200];
				strcpy(cs,pItem->cText);
				if (cs[0])
				{
					HDC hDC = GetDC(hMainWnd);
					HFONT pft = hMenuFont ? hMenuFont : hGuiFont;
					HFONT of = (HFONT)SelectObject(hDC,pft);
					SIZE osz;
					//õַά
					DWORD dwSZHW = GetTabbedTextExtent(hDC,cs,strlen(cs),0,NULL);
					osz.cx = LOWORD(dwSZHW);
					osz.cy = HIWORD(dwSZHW);
					if (pItem->iCmd == -4)//Menu tile
					{
						RECT rci;
						SetRectEmpty(&rci);
						DrawText(hDC,cs,strlen(cs),&rci, DT_CALCRECT|DT_TOP|DT_VCENTER|DT_SINGLELINE);
						lpm->itemHeight = szImage.cy + rci.bottom-rci.top;
						lpm->itemWidth = rci.right-rci.left;
					}
					else//Menu item
					{
						lpm->itemHeight = szImage.cy + 5;
						if (osz.cy > (int) lpm->itemHeight) 
							lpm->itemHeight = (int) osz.cy;
						lpm->itemWidth  = osz.cx + 2 + 15;
						lpm->itemWidth += lpm->itemHeight > (UINT) szImage.cx ? (UINT) lpm->itemHeight : (UINT) szImage.cx;
						//ͼbar
						lpm->itemWidth += m_iBackImageWidth;
					}
					SelectObject(hDC,of);
					ReleaseDC(hMainWnd,hDC);
				}
				else
				{
					lpm->itemHeight = szImage.cy + 5;
					lpm->itemWidth  = 100;
				}
			}
		}
		res = TRUE;
	}
	return res;
}

void CMenuSpawn::EnableMenuItems(HMENU hMenu,HWND pParent)
{
/*
	ASSERT(hMenu);
	ASSERT(pParent);

	int nItem = GetMenuItemCount(hMenu);

	while ((--nItem)>=0)
	{
		UINT itemId = GetMenuItemID(hMenu,nItem);
		if (itemId == (UINT) -1)
		{
			HMENU pops = GetSubMenu(hMenu,nItem);
			if (pops) EnableMenuItems(pops, pParent);
		}
		else
		{
			if (itemId != 0)
			{
				state.m_nID = itemId;
				pParent->OnCmdMsg(itemId, CN_UPDATE_COMMAND_UI, &state, NULL);
				state.DoUpdate(pParent, true);
			}
		}
		state.m_nIndex = nItem-1;
	}
*/
}


BOOL CMenuSpawn::SetFont(LOGFONT * lf)
{
	if(!lf)	return FALSE;
	if (hMenuFont) ::DeleteObject((HGDIOBJ)hMenuFont);
	hMenuFont = CreateFontIndirect(lf);
	return hMenuFont != NULL ? TRUE : FALSE;
}

BOOL CMenuSpawn::FindKeyboardShortcut(UINT nChar, UINT nFlags, HMENU hMenu, LRESULT & lRes)
{
	if(!hMenu) return FALSE;
	int nItem = GetMenuItemCount(hMenu);
	char csChar[100];
	wsprintf(csChar,"%c",(CHAR)nChar);
	//CString csChar((CHAR) nChar);
	//csChar.MakeLower();
	CharLowerBuff(csChar,strlen(csChar));
	while ((--nItem)>=0)
	{
		UINT itemId = GetMenuItemID(hMenu,nItem);
		if (itemId != 0)
		{
			MENUITEMINFO lpmi;
			ZeroMemory(&lpmi, sizeof(MENUITEMINFO));
			lpmi.cbSize = sizeof(MENUITEMINFO);
			lpmi.fMask = MIIM_DATA|MIIM_TYPE;
			if(!hMenu) return FALSE;
			if (GetMenuItemInfo(hMenu, nItem, TRUE, &lpmi))
			{
				if (lpmi.fType&MFT_OWNERDRAW)
				{
					SpawnItem * si = (SpawnItem *) lpmi.dwItemData;
					if (si)
					{
						char csItem[200];
						strcpy(csItem,si->cText);
						CharLowerBuff(csItem,strlen(csItem));
						//CString csItem(si->cText);
						//csItem.MakeLower();
						char *strstr1 = strstr(csItem,"&");
						//int iAmperIdx = csItem.Find('&');
						//if (iAmperIdx >= 0)
						if(strstr1)
						{
							//csItem = csItem.Mid(iAmperIdx + 1, 1);
							//csItem[0] = strstr1[1];
							if (strstr1[1] == csChar[0])
							{
								lRes = MAKELONG((WORD)nItem, 2);
								return TRUE;
							}
						}
					}
				}
			}
		}
	}
	return FALSE;
}

void CMenuSpawn::SetTextColor(const COLORREF crNormal, const COLORREF crSelected)
{
	crMenuText	= crNormal;
	crMenuTextSel	= crSelected;
}

VOID CMenuSpawn::SetBackBitmap(const int iRes)
{
	if(bmpBack)
		DeleteObject(bmpBack);
	bmpBack = NULL;
	bmpBack = LoadBitmap(hInst,MAKEINTRESOURCE(iRes));
	bBackBitmap = TRUE;
}

VOID CMenuSpawn::SetBackBitmap(const int iRes, COLORREF crBackColor)
{
	if(bmpBack) 
		DeleteObject(bmpBack);
	bmpBack = NULL;
	COLORMAP cMap1 = { crBackColor, cr3dFace };
	bmpBack = CreateMappedBitmap(hInst,iRes, 0, &cMap1, 1);
	bBackBitmap = TRUE;
}

BOOL CMenuSpawn::GetMenuItemText(LPSTR csText,INT nSize,HMENU hMenu, const INT cmd,BOOL bByPos)
{
	if(!hMenu) return FALSE;

	UINT itemId = bByPos ? GetMenuItemID(hMenu,cmd) : cmd;
	if (itemId != 0)
	{
		MENUITEMINFO lpmi;
		ZeroMemory(&lpmi, sizeof(MENUITEMINFO));
		lpmi.cbSize = sizeof(MENUITEMINFO);
		lpmi.fMask = MIIM_DATA|MIIM_TYPE;
		if(!hMenu) return FALSE;
		if (GetMenuItemInfo(hMenu, cmd, bByPos, &lpmi))
		{
			if (lpmi.fType&MFT_OWNERDRAW)
			{
				SpawnItem * si = (SpawnItem *) lpmi.dwItemData;
				if (si)
				{
					strcpy(csText,si->cText);
					return TRUE;
				}
			}
			else
			{
				return GetMenuString(hMenu,cmd,csText,nSize,bByPos ? MF_BYPOSITION : MF_BYCOMMAND) > 0;
			}
		}
	}
	return FALSE;
}

BOOL CMenuSpawn::IsSpawnMenu(HMENU hMenu, const INT iItem, const BOOL bByPos)
{
	if(!hMenu) return FALSE;

	UINT itemId = bByPos ? GetMenuItemID(hMenu,iItem) : iItem;
	if (itemId != 0)
	{
		MENUITEMINFO lpmi;
		ZeroMemory(&lpmi, sizeof(MENUITEMINFO));
		lpmi.cbSize = sizeof(MENUITEMINFO);
		lpmi.fMask = MIIM_DATA|MIIM_TYPE;
		if(!hMenu) return FALSE;
		if (GetMenuItemInfo(hMenu,iItem, bByPos, &lpmi))
		{
			if (lpmi.fType&MFT_OWNERDRAW)
			{
				SpawnItem * si = (SpawnItem *) lpmi.dwItemData;
				if (si) return TRUE;
			}
		}
	}
	return FALSE;
}
//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
#define MENU_TIMER_CODE	1000
BOOL	MenuPreTranslateMessage(
	CMenuSpawn& cSpawn,HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case WM_CREATE:
			SetTimer(hWnd,MENU_TIMER_CODE,100,NULL);
			break;
		case WM_TIMER:
			if (!cSpawn.bMenuLooping && wParam == MENU_TIMER_CODE)
			{
				POINT pt;
				GetCursorPos(&pt);
				LRESULT res = SendMessage(hWnd,WM_NCHITTEST,0, MAKELONG(pt.x, pt.y));
			}
			break;
		case WM_DRAWITEM:
			if (cSpawn.DrawItem((LPDRAWITEMSTRUCT)lParam))
				return TRUE;
			break;
		case WM_MEASUREITEM:
			if (cSpawn.MeasureItem(hWnd,(LPMEASUREITEMSTRUCT)lParam))
				return TRUE;
			break;
		case WM_INITMENU:
			{
			cSpawn.RemapMenu((HMENU)wParam);
			break;
			}
/*		case WM_INITMENUPOPUP:
			{
			HMENU hMenu = (HMENU)wParam;
			CMenu MyMenu;
			MyMenu.Attach(hMenu);
			cSpawn.RemapMenu(&MyMenu);
			hMenu = MyMenu.Detach();
			break;
			}
*/		case WM_MENUCHAR:
			{
				LRESULT lRes;
				if (cSpawn.FindKeyboardShortcut((TCHAR)LOWORD(wParam),(UINT)HIWORD(wParam),(HMENU) lParam,lRes)) 
					return TRUE;
				break;
			}
		case WM_NCHITTEST:
			{
				POINT point;
				point.x = LOWORD(lParam);
				point.y = HIWORD(lParam);
				UINT res = DefWindowProc(hWnd, message, wParam, lParam);;
				if (!cSpawn.bMenuLooping && res == HTMENU)
				{
					HMENU hMenu = GetMenu(hWnd);
					if(!hMenu) return 1;
					
					UINT mifp = MenuItemFromPoint(hWnd, hMenu, point);
					if (mifp != (UINT) -1)
					{
						RECT rc, wrc;
						if (GetMenuItemRect(hWnd,hMenu, mifp, &rc))
						{
							if (cSpawn.IsSpawnMenu(hMenu, mifp, TRUE))
							{
								GetWindowRect(hWnd,&wrc);
								rc.top -= wrc.top;
								rc.bottom -= wrc.top;
								rc.left -= wrc.left;
								rc.right -= wrc.left;
								if (EqualRect(&cSpawn.oldMenuBarRc,&rc) || IsRectEmpty(&cSpawn.oldMenuBarRc))
								{
									HDC hDC = GetWindowDC(hWnd);
									if (!(IsRectEmpty(&cSpawn.oldMenuBarRc)))
									{
										MyDraw3dRect(hDC,&cSpawn.oldMenuBarRc,cSpawn.crMenu, cSpawn.crMenu);
									}
									MyDraw3dRect(hDC,&rc,cSpawn.cr3dHilight, cSpawn.cr3dShadow);
									cSpawn.oldMenuBarRc = rc;
									ReleaseDC(hWnd,hDC);
								}
							}
							else
							{
								if (!(IsRectEmpty(&cSpawn.oldMenuBarRc)))
								{
									HDC hDC = GetWindowDC(hWnd);
									MyDraw3dRect(hDC,&cSpawn.oldMenuBarRc,cSpawn.crMenu, cSpawn.crMenu);
									SetRect(&cSpawn.oldMenuBarRc,0,0,0,0);
									ReleaseDC(hWnd,hDC);
								}
							}
						}
					}
				}
				else
				{
					if (!(IsRectEmpty(&cSpawn.oldMenuBarRc)))
					{
						HDC hDC = GetWindowDC(hWnd);
						MyDraw3dRect(hDC,&cSpawn.oldMenuBarRc,cSpawn.crMenu, cSpawn.crMenu);
						SetRect(&cSpawn.oldMenuBarRc,0,0,0,0);
						ReleaseDC(hWnd,hDC);
					}
				}
				return res;
				break;
			}
		case WM_ENTERMENULOOP:
				if (!(IsRectEmpty(&cSpawn.oldMenuBarRc)))
				{
					HDC hDC = GetWindowDC(hWnd);
					MyDraw3dRect(hDC,&cSpawn.oldMenuBarRc,cSpawn.crMenu, cSpawn.crMenu);
					SetRect(&cSpawn.oldMenuBarRc,0,0,0,0);
					ReleaseDC(hWnd,hDC);
				}
				cSpawn.bMenuLooping = TRUE;
			break;
		case WM_EXITMENULOOP:
				cSpawn.bMenuLooping = FALSE;
			break;
	}
	return FALSE;
}