#ifdef __GNUC__
#pragma implementation
#endif

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
#include "gmem.h"
#include "GString.h"
#include "Object.h"
#include "Stream.h"
#include "GfxState.h"
#include "GfxFont.h"
#include "Error.h"
//#include "Params.h"
#include "PBMOutputDev.h"
#include "config.h"

#include "BmpProc.h"
#include "pdfrlib.h"
CBmpProc	m_hDIB;

double get_ang(double mtx[6]);

//////////////////////////////////////////////////////////////////////////////////
int _XPutImage(Display* a,Drawable hClipRegion,
	       int bitcount,
	       COLORREF maskColor,
	       XImage*	f	/* image */,
	       int	g		/* src_width */,
	       int	h		/* src_height */,
	       int	i		/* dest_x */,
	       int	j		/* dest_y */,
	       unsigned int k	/* dest_width */,
	       unsigned int l	/* dest_height */,
	       LPSTEXT  pText
);
////////////////////////////////////////////////////////////////////

#define xoutRound(x) ((int)(x + 0.5))
#define PI 3.1415926535
#define RADIAN(angle) ((angle)*PI/180.0) //convert angle to radian
			
//------------------------------------------------------------------------

PBMOutputDev *PBMOutputDev::makePBMOutputDev(char *displayName,
					     char *fileRootA) 
{
	Display *displayA;
	Pixmap pixmapA;
	Window dummyWinA;
	int screenA;
	int invertA;
	unsigned long black, white;
	PBMOutputDev *out;
	
	if (!(displayA = XOpenDisplay(displayName))) 
	{
		fprintf(stderr, "Couldn't open display '%s'\n", displayName);
		exit(1);
	}
	screenA = DefaultScreen(displayA);
	black = BlackPixel(displayA, screenA);
	white = WhitePixel(displayA, screenA);
	if ((black & 1) == (white & 1)) {
		fprintf(stderr, "Weird black/white pixel colors\n");
		XCloseDisplay(displayA);
		return NULL;
	} 
	invertA = (white & 1) == 1 ? 0xff : 0x00;
	
	dummyWinA = XCreateSimpleWindow(displayA, RootWindow(displayA, screenA),
		0, 0, 1, 1, 0,
		black, white);
	pixmapA = XCreatePixmap(displayA, dummyWinA, 1, 1, 1);
	out = new PBMOutputDev(displayA, screenA, pixmapA, dummyWinA,
		invertA, fileRootA);
//	out->startDoc();
	return out;
}

void PBMOutputDev::killPBMOutputDev(PBMOutputDev *out) 
{
	Display *displayA;
	Pixmap pixmapA;
	Window dummyWinA;
	
	displayA = out->display;
	pixmapA = out->pixmap;
	dummyWinA = out->dummyWin;
	
	delete out;
	
	// these have to be done *after* the XOutputDev (parent of the
	// PBMOutputDev) is deleted, since XOutputDev::~XOutputDev() needs
	// them
	XFreePixmap(displayA, pixmapA);
	XDestroyWindow(displayA, dummyWinA);
	XCloseDisplay(displayA);
}

PBMOutputDev::PBMOutputDev(Display *displayA, int screenA,
			   Pixmap pixmapA, Window dummyWinA,
			   int invertA, char *fileRootA):
		XOutputDev(displayA, pixmapA, 1,
			DefaultColormap(displayA, screenA),
			WhitePixel(displayA, DefaultScreen(displayA)),
			gFalse, 1)
{
	display = displayA;
	screen = screenA;
	pixmap = pixmapA;
	dummyWin = dummyWinA;
	invert = invertA;
	fileRoot = fileRootA;
	fileName = (char *)gmalloc(strlen(fileRoot) + 20);

	dumpJPEG = gTrue;
	imgNum = 0;
}

PBMOutputDev::~PBMOutputDev() 
{
	gfree(fileName);
}

void PBMOutputDev::startPage(int pageNum, GfxState *state) 
{
	curPage = pageNum;
	width = (int)(state->getPageWidth() + 0.5);
	height = (int)(state->getPageHeight() + 0.5);

	m_hDIB.Clear();
	m_hDIB.Create(width,height,24);
	CRect rect = m_hDIB.Rect();
	m_hDIB.Fill(RGB(255,255,255),&rect);

	XFreePixmap(display, pixmap);
	pixmap = XCreatePixmap(display, dummyWin, width, height, 1);
	setPixmap(pixmap, width, height);
	XOutputDev::startPage(pageNum, state);
}

void PBMOutputDev::endPage() 
{
	//sprintf(fileName,"%s%03d.bmp",fileRoot,curPage);
	//m_hDIB.Save(fileName);
	//printf("Page: %d\n",curPage);
	XOutputDev::endPage();
}
static BYTE mask[8]={0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
inline BYTE GetPixelFromBmp2(int x, int y,int m_nScanWidth,LPBYTE lpData2)
{
	int OldByteNo = y*m_nScanWidth+x/8;	//ڼֽ
	int OldBitNo  = x%8;			//ڼλ
	if(lpData2[OldByteNo] & mask[OldBitNo])
		return 1;
	else 
		return 0;
}
VOID FlipImage(int width,int height,LPBYTE lpDIB32Image)
{
	if(lpDIB32Image == NULL)
		return;
	INT rowBytes = width*4;
	if(rowBytes <= 0) return;
	LPBYTE lpData = new BYTE[rowBytes];
	if(lpData == NULL)
		return;
	INT m_nHeight = height;
	INT m_nHeight_2 = m_nHeight/2;
	for(register int i = 0; i < m_nHeight_2; i++)
	{
		LPBYTE lpT = lpDIB32Image+i*rowBytes;
		LPBYTE lpB = lpDIB32Image+(m_nHeight-i-1)*rowBytes;
		memcpy(lpData,lpT,rowBytes);
		memcpy(lpT,lpB,rowBytes);
		memcpy(lpB,lpData,rowBytes);
	}
	delete []lpData;
}
VOID MirrorImage(int width,int height,LPBYTE lpDIB32Image)
{
	if(lpDIB32Image == NULL)
		return;
	INT rowBytes = width*4;
	if(rowBytes <= 0) 
		return;
	INT m_nHeight = height;
	INT m_nWidth = width;
	INT m_nWidth_2 = m_nWidth/2;
	for(int i = 0; i < m_nHeight; i++)
	{
		LPBYTE lpT = lpDIB32Image+i*rowBytes;
		LPDWORD lpDW = (LPDWORD)lpT;
		DWORD temp;
		for(int j = 0; j < m_nWidth_2; j++)
		{
			temp = lpDW[j];
			lpDW[j] = lpDW[m_nWidth-j-1];
			lpDW[m_nWidth-j-1] = temp;
		}
	}
}
extern VOID WINAPI DestroyRotateImage(COLORREF *pDst,SIZE sDst);
extern COLORREF * WINAPI RotateImage(COLORREF* lpbit,SIZE m_Size,double Angle,SIZE *sDst,COLORREF bg);
LPBYTE Rotate32Image(int Angle,int width,int height,int m_nScanWidth, 
		     LPBYTE lpData32,SIZE &sDst)
{
	SIZE m_Size;
	m_Size.cx = width;
	m_Size.cy = height;
	
	COLORREF *pDst = RotateImage(
		(COLORREF*)lpData32,
		m_Size,
		Angle,
		&sDst,
		RGB(255,255,255)
		);
	return (LPBYTE)pDst;
}
LPBYTE GetPixel32(int x, int y,int m_nWidth,int m_nHeight, LPBYTE lpDibBits)
{
	if (x >= m_nWidth || x < 0 || y >= m_nHeight || y < 0)
		return FALSE;
	BYTE* pData = lpDibBits + y*m_nWidth*4 + x*4;
	return pData;
}
BOOL SetPixel32(int x, int y,int m_nWidth,int m_nHeight,LPBYTE lpSrcData,LPBYTE lpDibBits)
{
	if (x >= m_nWidth || x < 0 || y >= m_nHeight || y < 0)
		return FALSE;
	BYTE* pDataDest = lpDibBits + y*m_nWidth*4 + x*4;
	memcpy(pDataDest,lpSrcData,4);
	return TRUE;
}
void PBMOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
				   int width, int height, GBool invert,
				   GBool inlineImg) 
{
	double *ctm;
	GBool rot;
	double xScale, yScale, xShear, yShear;
	int tx, ty, scaledWidth, scaledHeight, xSign, ySign;
	int ulx, uly, llx, lly, urx, ury, lrx, lry;
	int ulx1, uly1, llx1, lly1, urx1, ury1, lrx1, lry1;
	int bx0, by0, bx1, by1, bw, bh;
	int cx0, cy0, cx1, cy1, cw, ch;
	int x, y;
	int i, j;
	
	// get CTM, check for singular matrix
	ctm = state->getCTM();
	if (fabs(ctm[0] * ctm[3] - ctm[1] * ctm[2]) < 0.000001) 
	{
		error(-1, "Singular CTM in drawImage");
		if (inlineImg) 
		{
			j = height * ((width + 7) / 8);
			str->reset();
			for (i = 0; i < j; ++i) 
			{
				str->getChar();
			}
		}
		return;
	}
	
	// compute scale, shear, rotation, translation parameters
	rot = fabs(ctm[1]) > fabs(ctm[0]);
	if (rot) {
		xScale = -ctm[1];
		yScale = -ctm[2] + (ctm[0] * ctm[3]) / ctm[1];
		xShear = ctm[3] / yScale;
		yShear = -ctm[0] / ctm[1];
	} else {
		xScale = ctm[0];
		yScale = -ctm[3] + (ctm[1] * ctm[2]) / ctm[0];
		xShear = -ctm[2] / yScale;
		yShear = ctm[1] / ctm[0];
	}
	tx = xoutRound(ctm[2] + ctm[4]);
	ty = xoutRound(ctm[3] + ctm[5]);
	// use ceil() to avoid gaps between "striped" images
	scaledWidth = (int)ceil(fabs(xScale));
	xSign = (xScale < 0) ? -1 : 1;
	scaledHeight = (int)ceil(fabs(yScale));
	ySign = (yScale < 0) ? -1 : 1;
	
	// compute corners in device space
	ulx1 = 0;
	uly1 = 0;
	urx1 = xSign * (scaledWidth - 1);
	ury1 = xoutRound(yShear * urx1);
	llx1 = xoutRound(xShear * ySign * (scaledHeight - 1));
	lly1 = ySign * (scaledHeight - 1) + xoutRound(yShear * llx1);
	lrx1 = xSign * (scaledWidth - 1) +
		xoutRound(xShear * ySign * (scaledHeight - 1));
	lry1 = ySign * (scaledHeight - 1) + xoutRound(yShear * lrx1);
	if (rot) {
		ulx = tx + uly1;    uly = ty - ulx1;
		urx = tx + ury1;    ury = ty - urx1;
		llx = tx + lly1;    lly = ty - llx1;
		lrx = tx + lry1;    lry = ty - lrx1;
	} else {
		ulx = tx + ulx1;    uly = ty + uly1;
		urx = tx + urx1;    ury = ty + ury1;
		llx = tx + llx1;    lly = ty + lly1;
		lrx = tx + lrx1;    lry = ty + lry1;
	}
	
	// bounding box:
	//   (bx0, by0) = upper-left corner
	//   (bx1, by1) = lower-right corner
	//   (bw, bh) = size
	bx0 = (ulx < urx) ? (ulx < llx) ? (ulx < lrx) ? ulx : lrx
		: (llx < lrx) ? llx : lrx
		: (urx < llx) ? (urx < lrx) ? urx : lrx
		: (llx < lrx) ? llx : lrx;
	bx1 = (ulx > urx) ? (ulx > llx) ? (ulx > lrx) ? ulx : lrx
		: (llx > lrx) ? llx : lrx
		: (urx > llx) ? (urx > lrx) ? urx : lrx
		: (llx > lrx) ? llx : lrx;
	by0 = (uly < ury) ? (uly < lly) ? (uly < lry) ? uly : lry
		: (lly < lry) ? lly : lry
		: (ury < lly) ? (ury < lry) ? ury : lry
		: (lly < lry) ? lly : lry;
	by1 = (uly > ury) ? (uly > lly) ? (uly > lry) ? uly : lry
		: (lly > lry) ? lly : lry
		: (ury > lly) ? (ury > lry) ? ury : lry
		: (lly > lry) ? lly : lry;
	bw = bx1 - bx0 + 1;
	bh = by1 - by0 + 1;
	
	// Bounding box clipped to pixmap, i.e., "valid" rectangle:
	//   (cx0, cy0) = upper-left corner of valid rectangle in Pixmap
	//   (cx1, cy1) = upper-left corner of valid rectangle in XImage
	//   (cw, ch) = size of valid rectangle
	// These values will be used to transfer the XImage from/to the
	// Pixmap.
	cw = (bx1 >= pixmapW) ? pixmapW - bx0 : bw;
	if (bx0 < 0) {
		cx0 = 0;
		cx1 = -bx0;
		cw += bx0;
	} else {
		cx0 = bx0;
		cx1 = 0;
	}
	ch = (by1 >= pixmapH) ? pixmapH - by0 : bh;
	if (by0 < 0) {
		cy0 = 0;
		cy1 = -by0;
		ch += by0;
	} else {
		cy0 = by0;
		cy1 = 0;
	}
	
	// check for tiny (zero width or height) images
	// and off-page images
	if (scaledWidth <= 0 || scaledHeight <= 0 || cw <= 0 || ch <= 0) 
	{
		if (inlineImg) {
			j = height * ((width + 7) / 8);
			str->reset();
			for (i = 0; i < j; ++i) {
				str->getChar();
			}
		}
		return;
	}
//////////////////////////////////////////////////////////////////////////////
//Add for get image position and size, and rotate angle
	int x0, y0;			// top left corner of image
	int w0, h0, w1, h1;		// size of image
	double wt, ht, xt, yt;
	GBool rotate, xFlip, yFlip;
	state->transform(0, 0, &xt, &yt);
	state->transformDelta(1, 1, &wt, &ht);
	if (wt > 0) {
		x0 = xoutRound(xt);
		w0 = xoutRound(wt);
	} else {
		x0 = xoutRound(xt + wt);
		w0 = xoutRound(-wt);
	}
	if (ht > 0) {
		y0 = xoutRound(yt);
		h0 = xoutRound(ht);
	} else {
		y0 = xoutRound(yt + ht);
		h0 = xoutRound(-ht);
	}
	state->transformDelta(1, 0, &xt, &yt);
	rotate = fabs(xt) < fabs(yt);
	if (rotate) 
	{
		w1 = h0;
		h1 = w0;
		xFlip = ht < 0;
		yFlip = wt > 0;
	}
	else 
	{
		w1 = w0;
		h1 = h0;
		xFlip = wt < 0;
		yFlip = ht > 0;
	}
//////////////////////////////////////////////////////////////////////////////
	GfxRGB rgb;
	state->getFillRGB(&rgb);
	//ǰɫΪ趨ɫΪ͸ɫԱ⽫ǰΪɫ
	COLORREF m_maskColor = RGB((int)(rgb.r * 65535)>>8,
		(int)(rgb.g * 65535)>>8,
		(int)(rgb.b * 65535)>>8
		);
	if(m_maskColor == RGB(255,255,255))
		m_maskColor = RGB(255,255,240);

	int c;
	// dump JPEG file
	if (dumpJPEG && str->getKind() == strDCT) 
	{
		ASSERT(FALSE);
		FILE *f;
		// open the image file
		sprintf(fileName, "%s-%03d.jpg", fileRoot, imgNum);
		++imgNum;
		if (!(f = fopen(fileName, "wb"))) 
		{
			error(-1, "Couldn't open image file '%s'", fileName);
			return;
		}
		
		// initialize stream
		str = ((DCTStream *)str)->getRawStream();
		str->reset();
		
		// copy the stream
		while ((c = str->getChar()) != EOF)
			fputc(c, f);
		
		fclose(f);
		// dump PBM file
	} 
	else 
	{
		int  rowbytes = (width*1+31)/32*4;
		LPBYTE pData = (LPBYTE)malloc(rowbytes*height);
		LPBYTE pData2;
		// initialize stream
		str->reset();
		// for each line...
		for (y = 0; y < height; ++y)
		{
			if(yFlip == gFalse)
				pData2 = pData+(height-y-1)*rowbytes;
			else
				pData2 = pData+y*rowbytes;
			// write the line
			int width1 = ((width + 7) / 8);
			for (x = 0; x < width1; ++x)
			{
				if((c = str->getChar()) == EOF)
					break;
				if(!invert)
				{
					*pData2 = c;
					for(int ii = 0; ii < 8; ii++)
					{
						if(*pData2 & mask[ii])
							*pData2 &= ~mask[ii];
						else
							*pData2 |= mask[ii];
					}
					*pData2++;
				}
				else
					*pData2++ = c;
			}
		}

		//ֵͼΪ32λɫͼ
		int  rowbytes32 = (width*32+31)/32*4;
		LPBYTE pData32 = (LPBYTE)malloc(rowbytes32*height);
		memset(pData32,255,rowbytes32*height);
		BYTE rr = GetRValue(m_maskColor);
		BYTE gg = GetGValue(m_maskColor);
		BYTE bb = GetBValue(m_maskColor);
		for (y = 0; y < height; ++y)
		{
			LPBYTE pData2_32 = pData32+y*rowbytes32;
			for (x = 0; x < width; ++x)
			{
				if(GetPixelFromBmp2(x,y,rowbytes,pData))
				{//ǰɫΪɫ
					*pData2_32++ = bb;
					*pData2_32++ = gg;
					*pData2_32++ = rr;
					*pData2_32++ = 00;
				}
				else
				{//ɫΪ͸״̬ٶɫΪ͸ɫ
					*pData2_32++ = 255;
					*pData2_32++ = 255;
					*pData2_32++ = 255;
					*pData2_32++ = 00;
				}
			}
		}
		free(pData);

		BOOL m_bUserFree = FALSE;
		SIZE sDst;
		if (rotate)
		{
			LPBYTE pTmpData = Rotate32Image(-90,width,height,
				rowbytes32,pData32,sDst);
			if(pTmpData)
			{
				free(pData32);
				width  = sDst.cx;
				height = sDst.cy;
				pData32 = pTmpData;
				m_bUserFree = TRUE;
			}
		}
//		if(xFlip)
//			MirrorImage(width,height,pData32);
		if(yFlip)
			FlipImage(width,height,pData32);
		_XPutImage(NULL,(unsigned long)clipRegion,32,
			m_maskColor,//ǰɫ
			(XImage *)pData32,
			width/* src_width */,
			height/* src_height */,
			cx0/* dest_x */,
			cy0/* dest_y */,
			cw/* dest_width */,
			ch/* dest_height */,
			NULL);
		if(m_bUserFree)
			DestroyRotateImage((COLORREF*)pData32, sDst);
		else
			free(pData32);
	}
}

void PBMOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
			       int width, int height,
			       GfxImageColorMap *colorMap, int *maskColors,GBool inlineImg) 
{
	ImageStream *imgStr;
	Guchar pixBuf[4];
	GfxRGB rgb;
	int x, y,k;
	
	int nComps, nVals, nBits;
	GBool dither;
	double xt, yt;
	int ulx, uly, llx, lly, urx, ury, lrx, lry;
	int hx, hy, i, j;
	int bx0, by0, bx1, by1, bw, bh;
	int cx0, cy0, cx1, cy1, cw, ch;

//	ASSERT(maskColors == NULL);
	// image parameters
	nComps = colorMap->getNumPixelComps();
	nVals = width * nComps;
	nBits = colorMap->getBits();
	dither = nComps > 1 || nBits > 1;
	
	// corners in device space
	state->transform(0, 0, &xt, &yt);
	llx = xoutRound(xt);
	lly = xoutRound(yt);
	state->transform(0, 1, &xt, &yt);
	ulx = xoutRound(xt);
	uly = xoutRound(yt);
	state->transform(1, 0, &xt, &yt);
	lrx = xoutRound(xt);
	lry = xoutRound(yt);
	state->transform(1, 1, &xt, &yt);
	urx = xoutRound(xt);
	ury = xoutRound(yt);
	
	// horizontal traversal
	hx = urx - ulx;
	if (abs(lrx - llx) < abs(hx)) {
		hx = lrx - llx;
	}
	hy = ury - uly;
	if (abs(lry - lly) < abs(hy)) {
		hy = lry - lly;
	}
	
	// bounding box:
	//   (bx0, by0) = upper-left corner
	//   (bx1, by1) = lower-right corner
	//   (bw, bh) = size
	bx0 = bx1 = ulx;
	if (llx < bx0) {
		bx0 = llx;
	} else if (llx > bx1) {
		bx1 = llx;
	}
	if (urx < bx0) {
		bx0 = urx;
	} else if (urx > bx1) {
		bx1 = urx;
	}
	if (lrx < bx0) {
		bx0 = lrx;
	} else if (lrx > bx1) {
		bx1 = lrx;
	}
	by0 = by1 = uly;
	if (lly < by0) {
		by0 = lly;
	} else if (lly > by1) {
		by1 = lly;
	}
	if (ury < by0) {
		by0 = ury;
	} else if (ury > by1) {
		by1 = ury;
	}
	if (lry < by0) {
		by0 = lry;
	} else if (lry > by1) {
		by1 = lry;
	}
	bw = bx1 - bx0 + 1;
	bh = by1 - by0 + 1;
	
	// Bounding box clipped to pixmap, i.e., "valid" rectangle:
	//   (cx0, cy0) = upper-left corner of valid rectangle in Pixmap
	//   (cx1, cy1) = upper-left corner of valid rectangle in XImage
	//   (cw, ch) = size of valid rectangle
	// These values will be used to transfer the XImage from/to the
	// Pixmap.
	if (bx1 >= pixmapW) {
		cw = pixmapW - bx0;
	} else {
		cw = bw;
	}
	if (bx0 < 0) {
		cx0 = 0;
		cx1 = -bx0;
		cw += bx0;
	} else {
		cx0 = bx0;
		cx1 = 0;
	}
	if (by1 >= pixmapH) {
		ch = pixmapH - by0;
	} else {
		ch = bh;
	}
	if (by0 < 0) {
		cy0 = 0;
		cy1 = -by0;
		ch += by0;
	} else {
		cy0 = by0;
		cy1 = 0;
	}
	
	// check for tiny (zero width or height) images
	// and off-page images
	if (cw <= 0 || ch <= 0) 
	{
		if (inlineImg) 
		{
			str->reset();
			j = height * ((nVals * nBits + 7) / 8);
			for (i = 0; i < j; ++i)
				str->getChar();
		}
		return;
	}
//////////////////////////////////////////////////////////////////////////////
//Add for get image position and size, and rotate angle
	int x0, y0;			// top left corner of image
	int w0, h0, w1, h1;		// size of image
	double wt, ht;
	GBool rotate, xFlip, yFlip;
	state->transform(0, 0, &xt, &yt);
	state->transformDelta(1, 1, &wt, &ht);
	if (wt > 0) {
		x0 = xoutRound(xt);
		w0 = xoutRound(wt);
	} else {
		x0 = xoutRound(xt + wt);
		w0 = xoutRound(-wt);
	}
	if (ht > 0) {
		y0 = xoutRound(yt);
		h0 = xoutRound(ht);
	} else {
		y0 = xoutRound(yt + ht);
		h0 = xoutRound(-ht);
	}
	state->transformDelta(1, 0, &xt, &yt);
	rotate = fabs(xt) < fabs(yt);
	if (rotate) {
		w1 = h0;
		h1 = w0;
		xFlip = ht < 0;
		yFlip = wt > 0;
	} else {
		w1 = w0;
		h1 = h0;
		xFlip = wt < 0;
		yFlip = ht > 0;
	}
//////////////////////////////////////////////////////////////////////////////
	{
		int  rowbytes = (width*32+31)/32*4;
		LPBYTE pData = (LPBYTE)malloc(rowbytes*height);
		LPBYTE pData2;
		
		// initialize stream
		imgStr = new ImageStream(str, width, colorMap->getNumPixelComps(),
			colorMap->getBits());
		imgStr->reset();
		
		// for each line...
		for (y = 0; y < height; ++y) 
		{
			if(yFlip == gFalse)
				pData2 = pData+(height-y-1)*rowbytes;
			else
				pData2 = pData+y*rowbytes;
			// write the line
			for (x = 0; x < width; ++x) 
			{
				imgStr->getPixel(pixBuf);
				colorMap->getRGB(pixBuf, &rgb);
				int b = (int)(rgb.b*65535+0.5);
				int g = (int)(rgb.g*65535+0.5);
				int r = (int)(rgb.r*65535+0.5);
				b = b>>8;
				g = g>>8;
				r = r>>8;
				if(r == 255 && g == 255 && b == 255)
					b = 250;
				if (maskColors)
				{
					for (k = 0; k < nComps; ++k) 
					{
						if (pixBuf[k] < maskColors[2*k] ||
							pixBuf[k] > maskColors[2*k]) 
						{
							*pData2++ = (unsigned char)b;
							*pData2++ = (unsigned char)g;
							*pData2++ = (unsigned char)r;
							*pData2++ = (unsigned char)0;
							break;
						}
						else
						{
							*pData2++ = 255;
							*pData2++ = 255;
							*pData2++ = 250;//transparence
							*pData2++ = 0;
						}
					}
				}
				else
				{
					*pData2++ = (unsigned char)b;
					*pData2++ = (unsigned char)g;
					*pData2++ = (unsigned char)r;
					*pData2++ = (unsigned char)0;
				}
			}
		}
		delete imgStr;

		//xueheng
		rotate = 0;

		BOOL m_bUserFree = FALSE;
		SIZE sDst;
		if(rotate)
		{
			double *pCTM = state->getCTM();
			double rotate1 = get_ang(pCTM);
			int rotate2 =  (double)(rotate1*57.295779+0.5); 
			LPBYTE pTmpData = Rotate32Image(
				(-1)*rotate2,width,height,
				rowbytes,pData,sDst);
			if(pTmpData)
			{
				free(pData);
				width  = sDst.cx;
				height = sDst.cy;
				pData = pTmpData;
				m_bUserFree = TRUE;
			}
		}
		else
		{
			if(xFlip)
				MirrorImage(width,height,pData);
		}

		_XPutImage(NULL,(unsigned long)clipRegion,32,
			0,
			(XImage *)pData,
			width/* src_width */,
			height/* src_height */,
			cx0/* dest_x */,
			cy0/* dest_y */,
			cw/* dest_width */,
			ch/* dest_height */,
			NULL);
		if(m_bUserFree)
			DestroyRotateImage((COLORREF*)pData, sDst);
		else
			free(pData);
	}
}


int GetWidth()
{
	return m_hDIB.Width();
}
int GetHeight()
{
	return m_hDIB.Height();
}
typedef float            Float;
#define  PS_ABS(a)    ((a)<0?(-(a)):(a))
#define  PSG_NEAR       .000001
/*@@*/   int pstrans(double *mtx, double x, double y, double *xp, double *yp,int dist)
{
   *xp = (float)(mtx[0]*x); if( mtx[2] ) *xp += (float)(mtx[2]*y);
   *yp = (float)(mtx[3]*y); if( mtx[1] ) *yp += (float)(mtx[1]*x);
   if( !dist ) { *xp += mtx[4]; *yp += mtx[5]; }
   return(0);
}

static int S_TextEmul(double *mtx)
{
	double xx, yy;
	double x, y;
	double bang, sang;
	/*
	*  Calculate skew angle
	*/
	/* we're still getting a skew on fairly small values of xx, yy 
	-0.0011 0.0007 tm
	*/
	x = mtx[1]; y = mtx[2];
	if( x < 0 ) x = -x;
	if( y < 0 ) y = -y;
	if( x < 0.0001 && y < 0.0001 ) 
		return(0);
	
	x = (double)1.; y = (double)0.;
	pstrans(mtx, x, y, &xx, &yy, 1);
	if( PS_ABS(xx) < PSG_NEAR && PS_ABS(yy) < PSG_NEAR ) 
		bang = (Float)0.;
	else 
		bang = (double)atan2(yy, xx);
	
	x = (double)0.; y = (double)1.;
	pstrans(mtx, x, y, &xx, &yy, 1);
	if( PS_ABS(xx)<PSG_NEAR && PS_ABS(yy)<PSG_NEAR ) 
		sang = (Float)0.;
	else 
		sang = (double)atan2(yy, xx);
	
	while( bang<sang ) 
		bang += (double)6.28318530718;
	if( PS_ABS((bang-sang)-1.5707963268)<PSG_NEAR ) 
		return(0);
	return(1);
}
double get_ang(double mtx[6])
{
	double xx,yy;
	double ang;
	double x = 1., y = 0.;
	pstrans(mtx, x, y, &xx, &yy, 1);
	if( PS_ABS(xx)<PSG_NEAR && PS_ABS(yy)<PSG_NEAR ) return(0.);
	yy = -yy;
	ang = (double)atan2(yy, xx);
	return(ang);
}
typedef struct { double x,y;   } FPNT;
double psdist(FPNT *pt1, FPNT *pt2)
{
   float  dist,dx,dy;

   dx   = pt2->x-pt1->x;
   dy   = pt2->y-pt1->y;
   if( dx==0. && dy==0. ) return(0.);
   dist = (float)sqrt(dx*dx+dy*dy);
   return((double)dist);
}
double get_mht2(double mtx[6],double x,double y)
{
	FPNT  p1, p2;
	double ht = 1.;
	p1.x = p1.y = 0.;
	pstrans(mtx, x, y, &p2.x, &p2.y, 1);
	if( PS_ABS(p2.x)>=PSG_NEAR || PS_ABS(p2.y)>=PSG_NEAR )
	{
		ht = psdist(&p1, &p2);
		if( ht<1. ) ht = 1.;
	}
	return(ht);
}
double get_mht(double mtx[6])
{
	double x = 0., y = 1000.;
	return get_mht2(mtx,x,y);
}
double get_mwd2(double mtx[6],double x,double y)
{
	FPNT  p1, p2;
	double wd = 1.;
	p1.x = p1.y = 0.;
	pstrans(mtx, x, y, &p2.x, &p2.y, 1);
	if( PS_ABS(p2.x)>=PSG_NEAR || PS_ABS(p2.y)>=PSG_NEAR )
	{
		wd = psdist(&p1, &p2);
		if( wd<1. ) wd = 1.;
	}
	return(wd);
}
double get_mwd(double mtx[6])
{
	double x = 1000., y = 0.;
	return get_mwd2(mtx,x,y);
}
void GetRotatedPos(double x,double y,double centerx,double centery,double* xp,double* yp,double angle)
{
	double _angle = angle;
	_angle = (float)RADIAN(_angle);
	double cosa = cos(_angle);
	double sina = sin(_angle);
	
	typedef struct tagPOINT2
	{
		double  x;
		double  y;
	} POINT2;

	POINT2 m_ptCenter;
	m_ptCenter.x = centerx;
	m_ptCenter.y = centery;
	double x1 = x;
	double y1 = y;
	*xp = (x1 * cosa - y1 * sina + m_ptCenter.x-m_ptCenter.x*cosa+m_ptCenter.y*sina);
	*yp = (x1 * sina + y1 * cosa + m_ptCenter.y-m_ptCenter.x*sina-m_ptCenter.y*cosa);

	return;
}
////////////////////////////////////////////////////////////////////////////////////////////////
//
//
//滭
//
//
////////////////////////////////////////////////////////////////////////////////////////////////
int XDrawLines(Display* a,Drawable clipRegion,GC color,XPoint* f,int nCountPoint,
	       int mode,int nLineWidth,int style)
{
	m_hDIB.SetClipRegion((HRGN)clipRegion);
	for(int i = 0; i < nCountPoint-1; i++)
	{
		m_hDIB.DrawLines(f[i].x,f[i].y,f[i+1].x,f[i+1].y,
			nLineWidth,(unsigned long)color,style);
	}
	m_hDIB.SetClipRegion((HRGN)NULL);
	return 0;
}
void MATRIX_Concat (double* self, double* mtx1, double* mtx2)
{
	double xx1, yy1, tx1, ty1;
	double xx2, yy2, xy2, yx2;
	double* se = self;
	double* m1 = mtx1;
	double* m2 = mtx2;
	
	xx1 = m1[0]; yy1 = m1[3]; tx1 = m1[4]; ty1 = m1[5];
	xx2 = m2[0]; yy2 = m2[3]; xy2 = m2[1]; yx2 = m2[2];
	if( (!m1[1]) && (!m1[2]) )
	{
		se[4] = tx1*xx2+m2[4];
		se[5] = ty1*yy2+m2[5];
		if( !xy2 ) se[1] = 0.;
		else { se[1] = xx1*xy2; se[5] += tx1*xy2; }
		se[0] = xx1*xx2;
		if( !yx2 ) se[2] = 0.;
		else { se[2] = yy1*yx2; se[4] += ty1*yx2; }
		se[3] = yy1*yy2;
	}
	else
	{
		se[0] = xx1*xx2+m1[1]*yx2;
		se[1] = xx1*xy2+m1[1]*yy2;
		se[2] = m1[2]*xx2+yy1*yx2;
		se[3] = m1[2]*xy2+yy1*yy2;
		se[4] = tx1*xx2+ty1*yx2+m2[4];
		se[5] = tx1*xy2+ty1*yy2+m2[5];
	}
}
int XDrawString(Display* a,Drawable clipRegion,GC color,int xxx,int yyy,
		int strwidth,int strheight,_Xconst char* szString,int nCharCount)
{
	GfxState *state = (GfxState *)a;
	if(state == NULL)
		return -1;

//xueheng add for find color,begin
	GfxRGB rgb;
	state->getFillRGB(&rgb);
	COLORREF m_clrFill = RGB((int)(rgb.r * 65535)>>8,
		(int)(rgb.g * 65535)>>8,
		(int)(rgb.b * 65535)>>8
		);
	state->getStrokeRGB(&rgb);
	COLORREF m_clrStroke = RGB((int)(rgb.r * 65535)>>8,
		(int)(rgb.g * 65535)>>8,
		(int)(rgb.b * 65535)>>8
		);
	int m_iRender = state->getRender();
	switch(m_iRender)
	{
	case 0://fill text
	case 1://stroke text
	case 2://fill and stroke text
	case 4://Fill text and add it to the clipping path
	case 5://Stroke text and add it to the clipping path
	case 6://Fill then stroke text and add it to the clipping path
		break;
	case 3://invisible
	case 7://Add text to the clipping path
		return 0;
	}

//xueheng add for find color,end
	int nsize  = (int)(state->getTransformedFontSize()+0.5);
	int nFontSizeWidth  = 0;
	int nFontSizeHeight = 0;

	int isBold = FALSE,isItalic = FALSE;
	BOOL isWMode16 = FALSE;
	double rotate = 0.0;

	char fontname[200];
	strcpy(fontname,"Courier");
	GfxFont *pFont = state->getFont();
	if(pFont)
	{
		GString *pszFont = pFont->getName();
		if(pszFont)
			strcpy(fontname,pszFont->getCString());

		isItalic = pFont->isItalic();
		isBold   = pFont->isBold();
		//for font italic
		//rotate += pFont->getItalicAngle();
		//double *m_pdbFontMatrix = state->getTextMat();
		//if(m_pdbFontMatrix && rotate == 0.0 && S_TextEmul(m_pdbFontMatrix))
		//	isItalic = gTrue;
		if(pFont->getWMode())
		{//
			isWMode16 = TRUE;
		}
		else
		{//
		}
		m_hDIB.SetCharSet(pFont->getWinCharSet());

		char *pSubStr;
		//Italic, Bold, or BoldItalic.
		if(	(pSubStr = strstr(fontname,",Italic")) != NULL
			|| ((pSubStr = strstr(fontname,"-Italic")) != NULL)
			|| ((pSubStr = strstr(fontname,"Italic")) != NULL)
			)
		{
			*pSubStr = '\0';
			isItalic = gTrue;
		}
		if(	(pSubStr = strstr(fontname,",Bold")) != NULL
			|| ((pSubStr = strstr(fontname,"-Bold")) != NULL)
			|| ((pSubStr = strstr(fontname,"Bold")) != NULL)
			|| ((pSubStr = strstr(fontname,"-Black")) != NULL)
			)
		{
			*pSubStr = '\0';
			isBold = gTrue;
		}
		//pseudoUniqueTag+PostScriptName͵Ӽ磺HOMMAG+MS-Mincho
		if((pSubStr = strstr(fontname,"+")) != NULL)
		{
			*pSubStr = '\0';
			strcpy(fontname,pSubStr+1);
		}
		if((pSubStr = strstr(fontname,"STHeiti")) != NULL)
			strcpy(fontname,"SimHei");
		if((pSubStr = strstr(fontname,"STSong")) != NULL)
			strcpy(fontname,"SimSun");
	}
//Add by xueheng for "PointText" at 2002/03/28,begin
	GList	*m_pListMarkedContent = &m_listBdcBmcMarkedContent;
	if(isWMode16 == FALSE && m_pListMarkedContent)
	{//
		int m_iListLength = m_pListMarkedContent->getLength();
		for(int i = 0; i < m_iListLength && isWMode16==FALSE; i++)
		{
			LPSBDC_BMC_MARKED_CONTENT pMarkContent = 
				(LPSBDC_BMC_MARKED_CONTENT)m_pListMarkedContent->get(i);
			if(pMarkContent == NULL)
				continue;
			if(pMarkContent->numArgs != 2)
				continue;
			if(pMarkContent->args[0].isName("PointText") 
				&& pMarkContent->args[1].isDict())
			{
				Dict *m_pDictPointText = pMarkContent->args[1].getDict();
				if(m_pDictPointText)
				{
					Object obj1;
					if(m_pDictPointText->lookupNF("Vertical",&obj1))
					{
						if(obj1.isBool() && obj1.getBool() == gTrue)
						{
							isWMode16 = TRUE;
						}
					}
					obj1.free();
				}
			}
		}
	}

//Add by xueheng for "PointText" at 2002/03/28,end
	{
		double m_nFontCTM[6];
		state->getFontTransMat(&m_nFontCTM[0],&m_nFontCTM[1],&m_nFontCTM[2],&m_nFontCTM[3]);
		m_nFontCTM[4] = m_nFontCTM[5] = 0.0;
		double *pCTM = &m_nFontCTM[0];
		if(pCTM)
		{
			//computer rotate matrix
			rotate = get_ang(pCTM);
			rotate =  (double)(rotate*57.295779);
			if(rotate != 0.0)
			{
			}
			//computer scale matrix
			double     wd = get_mwd(pCTM);
			double     ht = get_mht(pCTM);
			nFontSizeWidth  = wd/1000;
			nFontSizeHeight = ht/1000;
			if(rotate == 0.0 && S_TextEmul(pCTM))
				isItalic = gTrue;
		}
		//double *ctm = state->getCTM();
		//double *textMat = state->getTextMat();
		//double xxx = (textMat[4] * ctm[0] + textMat[5] * ctm[2]) + ctm[4];
		//double yyy = (textMat[4] * ctm[1] + textMat[5] * ctm[3]) + ctm[5];
		//f = xxx;
		//g = yyy;
	}

//XueHeng added to output text infomation

	{//ַַֽ
		m_hDIB.SetClipRegion((HRGN)clipRegion);
		m_hDIB.TextOutEx(xxx,yyy,
			szString,nCharCount,
			//fontname,nFontSizeWidth,nFontSizeHeight,
			fontname,strwidth,nFontSizeHeight,
			isBold,isItalic,m_clrFill,m_clrStroke,
			m_iRender,rotate,isWMode16,
			"SHIFT-JIS","EUC-JP");
		m_hDIB.SetClipRegion((HRGN)NULL);
	}
	return 0;
}
int _XPutImage(Display* a,Drawable clipRegion,int bitcount,
	      COLORREF  m_maskColor,
	      XImage*	f	/* image */,
	      int	g		/* src_width */,
	      int	h		/* src_height */,
	      int	i		/* dest_x */,
	      int	j		/* dest_y */,
	      unsigned int k	/* dest_width */,
	      unsigned int l	/* dest_height */,
	      LPSTEXT  pText
)
{
	m_hDIB.SetClipRegion((HRGN)clipRegion);
	m_hDIB.DrawImage((int)bitcount,m_maskColor,g,h,(LPBYTE)f,i,j,k,l,pText);
	m_hDIB.SetClipRegion((HRGN)NULL);
	return 0;
}
int XFillPolygon(
    Display* a		/* display */,
    Drawable clipRegion		/* d */,
    GC color			/* gc */,
    XPoint*	f	/* points */,
    int g			/* npoints */,
    int	shape		/* shape */,
    int	mode		/* mode */
)
{
	GfxState *state = (GfxState *)a;
	if(state == NULL)
		return -1;

	double m_dbFillOpacity = state->getFillOpacity();
	if(m_dbFillOpacity != 1.0)
	{
	}
	int nsize  = 1;//(int)(state->getTransformedLineWidth()+0.5);
	switch(mode)
	{
	case WindingRule:
		mode = WINDING;
		break;
	case EvenOddRule:
		mode = ALTERNATE;
		break;
	}
	m_hDIB.SetClipRegion((HRGN)clipRegion);
	m_hDIB.FillPolygon((LPPOINT)f,g,shape,mode,nsize,(unsigned long)color);
	m_hDIB.SetClipRegion((HRGN)NULL);
	return 0;
}
int _XXueSetRegion(HRGN hClipRgn)
{
	m_hDIB.ChangeClipRegion(hClipRgn);
	return 0;
}